package com.dc.standard.report.service.impl;


import com.dc.standard.report.entity.*;
import com.dc.standard.report.mapper.GuanbiaobaobiaoMapper;
import com.dc.standard.report.entity.ErpMaterial;
import com.dc.standard.report.entity.PdmProduct;
import com.dc.standard.report.service.IErpMaterialService;
import com.dc.standard.report.service.IPdmProductService;
import com.dc.standard.report.service.*;
import com.dc.standard.report.utils.DateUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;


import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;


import javax.annotation.Resource;
import java.util.HashSet;
import java.util.List;
import java.util.stream.Collectors;

@Slf4j
@Transactional(rollbackFor = Exception.class)
@Service
public class GuanbiaobaobiaoServiceImpl extends ServiceImpl<GuanbiaobaobiaoMapper, Guanbiaobaobiao> implements IGuanbiaobaobiaoService {
    @Resource
    private IErpMaterialService erpMaterialService;
    @Resource
    private IYsMaterialService ysMaterialService;
    @Resource
    private IMaterialService materialService;
    @Resource
    private IPdmProductService pdmProductService;
    @Resource
    private IYsOrderService ysOrderService;
    @Resource
    private IOrderService orderService;


    @Override
    public void statisticalData() {
        String now = DateUtils.getNow("yyyy/MM/dd");
        Guanbiaobaobiao materialData = getGuanbiaobaobiaoByType("物资");
        materialData.setGxrq(now);
        materialData(materialData);
        log.info("物资数据：${}", materialData);
        this.saveOrUpdate(materialData);
        Guanbiaobaobiao productData = getGuanbiaobaobiaoByType("型号");
        productData.setGxrq(now);
        productData(productData);
        log.info("型号数据：${}", productData);
        this.saveOrUpdate(productData);
    }

    private void productData(Guanbiaobaobiao guanbiaobaobiao) {
        guanbiaobaobiao.setZsjlx("型号");
        //累计接收数量
        List<PdmProduct> list = pdmProductService.lambdaQuery().list();
        guanbiaobaobiao.setLjjssl(list.size()+"");
        //本月接收数量
        List<PdmProduct> pdmProductListMonth = list.stream().filter(pdmProduct -> DateUtils.isYearAndMonth(pdmProduct.getCreateTime())).collect(Collectors.toList());
        guanbiaobaobiao.setByxfsl(pdmProductListMonth.size()+"");
        guanbiaobaobiao.setByjssl(pdmProductListMonth.size()+"");
        //累计应用主数量总量
        List<YsOrder> ysOrderList = ysOrderService.lambdaQuery().isNotNull(YsOrder::getPdmProductId).list();
        List<Order> orderList = orderService.lambdaQuery().isNotNull(Order::getPdmProductId).list();
        HashSet<String> pdmProductIds = new HashSet<>();
        pdmProductIds.addAll(ysOrderList.stream().map(YsOrder::getPdmProductId).collect(Collectors.toList()));
        pdmProductIds.addAll(orderList.stream().map(Order::getPdmProductId).collect(Collectors.toList()));
        guanbiaobaobiao.setLjyyzsjzl(pdmProductIds.size()+"");
        //本月应用数量
        List<YsOrder> ysOrderListMonth = ysOrderList.stream().filter(order -> DateUtils.isYearAndMonth(order.getCreateTime())).collect(Collectors.toList());
        List<Order> orderListMonth = orderList.stream().filter(order -> DateUtils.isYearAndMonth(order.getCreateTime())).collect(Collectors.toList());
        HashSet<String> monthPdmProductIds = new HashSet<>();
        monthPdmProductIds.addAll(ysOrderListMonth.stream().map(YsOrder::getPdmProductId).collect(Collectors.toList()));
        monthPdmProductIds.addAll(orderListMonth.stream().map(Order::getPdmProductId).collect(Collectors.toList()));
        guanbiaobaobiao.setByyysl(monthPdmProductIds.size()+"");
    }

    public void materialData(Guanbiaobaobiao guanbiaobaobiao) {
        guanbiaobaobiao.setZsjlx("物资");
        //累计接收数量
        List<ErpMaterial> list = erpMaterialService.lambdaQuery().list();
        guanbiaobaobiao.setLjjssl(list.size()+"");
        //本月接收数量
        List<ErpMaterial> erpMaterialListMonth = list.stream().filter(erpMaterial -> DateUtils.isYearAndMonth(erpMaterial.getCreateTime())).collect(Collectors.toList());
        guanbiaobaobiao.setByxfsl(erpMaterialListMonth.size()+"");
        guanbiaobaobiao.setByjssl(erpMaterialListMonth.size()+"");
        //累计应用主数量总量
        List<YsMaterial> ysMaterialList = ysMaterialService.list().stream().filter(material -> material.getDataSource().equals("0")).collect(Collectors.toList());
        List<Material> materialList = materialService.list().stream().filter(material -> material.getDataSource().equals("0") ).collect(Collectors.toList());
        HashSet<String> codes = new HashSet<>();
        codes.addAll(ysMaterialList.stream().map(YsMaterial::getCode).collect(Collectors.toList()));
        codes.addAll(materialList.stream().map(Material::getCode).collect(Collectors.toList()));
        guanbiaobaobiao.setLjyyzsjzl(codes.size()+"");
        //本月应用数量
        List<YsMaterial> ysMaterialListMonth = ysMaterialList.stream().filter(material -> DateUtils.isYearAndMonth(material.getCreateTime())).collect(Collectors.toList());
        List<Material> materialListMonth = materialList.stream().filter(material -> DateUtils.isYearAndMonth(material.getCreateTime())).collect(Collectors.toList());
        HashSet<String> monthCodes = new HashSet<>();
        monthCodes.addAll(ysMaterialListMonth.stream().map(YsMaterial::getCode).collect(Collectors.toList()));
        monthCodes.addAll(materialListMonth.stream().map(Material::getCode).collect(Collectors.toList()));
        guanbiaobaobiao.setByyysl(monthCodes.size()+"");
    }

    public Guanbiaobaobiao getGuanbiaobaobiaoByType(String type) {
        Guanbiaobaobiao guanbiaobaobiao = this.lambdaQuery().eq(Guanbiaobaobiao::getZsjlx, type).like(Guanbiaobaobiao::getGxrq, DateUtils.getMonth("yyyy/MM")).one();
        if (guanbiaobaobiao != null) {
            return guanbiaobaobiao;
        }
        return new Guanbiaobaobiao();
    }
}
