package com.dc.standard.report.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dc.standard.report.entity.*;
import com.dc.standard.report.mapper.HuoyueduMapper;
import com.dc.standard.report.service.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

@Transactional(rollbackFor = Exception.class)
@Service
@Slf4j
public class HuoyueduServiceImpl extends ServiceImpl<HuoyueduMapper, Huoyuedu> implements IHuoyueduService {
    @Autowired
    private IErpMaterialService erpMaterialService;
    @Autowired
    private IYsMaterialService ysMaterialService;
    @Autowired
    private IMaterialService materialService;
    @Autowired
    private IPdmProductService pdmProductService;
    @Autowired
    private IYsOrderService ysOrderService;
    @Autowired
    private IOrderService orderService;


    @Override
    public void statisticalData() {
        List<Huoyuedu> list = this.list();
        List<Huoyuedu> saveList = new ArrayList<Huoyuedu>();
        saveList.addAll(materialData(list));
        saveList.addAll(productDate(list));
        if (!saveList.isEmpty()) {
            this.saveBatch(saveList);
        }
    }

    private List<Huoyuedu> productDate(List<Huoyuedu> list) {
        String type = "型号";
        List<PdmProduct> pdmProducts = pdmProductService.lambdaQuery().list();
        if (CollectionUtil.isEmpty(pdmProducts)) {
            log.info("活跃度没有型号数据进行添加");
            return new ArrayList<>();
        }
        List<YsOrder> ysOrderList = ysOrderService.lambdaQuery().isNotNull(YsOrder::getPdmProductId).list();
        List<Order> orderList = orderService.lambdaQuery().isNotNull(Order::getPdmProductId).list();
        List<Huoyuedu> saveList = new ArrayList<Huoyuedu>();
        for (PdmProduct product : pdmProducts) {
            //获取 ysOrderList、orderList 的id 在 list中Zsjyyid 的数据
            for (YsOrder ysOrder : ysOrderList) {
                //如果 ysOrder id 和 list Zsjyyid 相同 并且 product ProductId 和 list 中Zsjid 相同 则跳过
                if (list.stream().anyMatch(item -> ysOrder.getId().equals(item.getZsjyyid()) && product.getProductId().equals(item.getZsjid()))) {
                    continue;
                }
                Huoyuedu huoyuedu = new Huoyuedu();
                huoyuedu.setZsjid(product.getProductId());
                huoyuedu.setZsjmc(product.getProductName());
                huoyuedu.setZsjlx(type);
                huoyuedu.setZsjyyid(ysOrder.getId());
                //这里可能是表名称
                huoyuedu.setBdmc("工号");
                huoyuedu.setStsyrq(DateUtil.format(ysOrder.getCreateTime(), "yyyy/MM/dd"));
                huoyuedu.setCzry(ysOrder.getCreateBy());
                saveList.add(huoyuedu);
            }
            for (Order order : orderList) {
                if (list.stream().anyMatch(item -> order.getId().equals(item.getZsjyyid()) && product.getProductId().equals(item.getZsjid()))) {
                    continue;
                }
                Huoyuedu huoyuedu = new Huoyuedu();
                huoyuedu.setZsjid(product.getProductId());
                huoyuedu.setZsjmc(product.getProductName());
                huoyuedu.setZsjlx(type);
                huoyuedu.setZsjyyid(order.getId());
                huoyuedu.setBdmc("工号");
                huoyuedu.setStsyrq(DateUtil.format(order.getCreateTime(), "yyyy/MM/dd"));
                huoyuedu.setCzry(order.getCreateBy());
                saveList.add(huoyuedu);
            }
        }
        return saveList;
    }

    private List<Huoyuedu> materialData(List<Huoyuedu> list) {
        String type = "物资";
        List<ErpMaterial> erpMaterialList = erpMaterialService.lambdaQuery().isNotNull(ErpMaterial::getMatnr).list();
        if (CollectionUtil.isEmpty(erpMaterialList)) {
            log.info("活跃度没有物料数据进行添加");
            return new ArrayList<>();
        }
        List<YsMaterial> ysMaterialList = ysMaterialService.lambdaQuery().eq(YsMaterial::getDataSource, "0").list();
        List<Material> materialList = materialService.lambdaQuery().eq(Material::getDataSource, "0").list();
        List<Huoyuedu> saveList = new ArrayList<Huoyuedu>();
        for (ErpMaterial erpMaterial : erpMaterialList){
            for (YsMaterial ysMaterial : ysMaterialList){
                if (list.stream().anyMatch(item -> ysMaterial.getId().equals(item.getZsjyyid()) && item.getZsjid().equals(erpMaterial.getMatnr()))) {
                    continue;
                }
                Huoyuedu huoyuedu = new Huoyuedu();
                huoyuedu.setZsjid(erpMaterial.getMatnr());
                huoyuedu.setZsjmc(ysMaterial.getMaterialLongDescription());
                huoyuedu.setZsjlx(type);
                huoyuedu.setZsjyyid(ysMaterial.getId());
                //可能是表名称
                huoyuedu.setBdmc("物料");
                huoyuedu.setStsyrq(DateUtil.format(ysMaterial.getCreateTime(), "yyyy/MM/dd"));
                huoyuedu.setCzry(ysMaterial.getCreateBy());
                saveList.add(huoyuedu);
            }
            for (Material material : materialList) {
                if (list.stream().anyMatch(item -> material.getId().equals(item.getZsjyyid()) && item.getZsjid().equals(erpMaterial.getMatnr()))) {
                    continue;
                }
                Huoyuedu huoyuedu = new Huoyuedu();
                huoyuedu.setZsjid(erpMaterial.getMatnr());
                huoyuedu.setZsjmc(material.getMaterialLongDescription());
                huoyuedu.setZsjlx(type);
                huoyuedu.setZsjyyid(material.getId());
                huoyuedu.setBdmc("物料");
                huoyuedu.setStsyrq(DateUtil.format(material.getCreateTime(), "yyyy/MM/dd"));
                huoyuedu.setCzry(material.getCreateBy());
                saveList.add(huoyuedu);
            }
        }
        return saveList;
    }

}
